@@ -2,7 +2,7 @@
 """makes the bridge between repository handlers and related cubicweb entities
 
 :organization: Logilab
-:copyright: 2007-2010 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+:copyright: 2007-2011 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 :contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
 """
 from __future__ import with_statement
@@ -73,7 +73,7 @@ def repository_handler(repoent):
             return repohdlr
 
 
-def import_content(session, commitevery=10):
+def import_content(session, commitevery=10, raise_on_error=False):
     """synchronize content of known vcs repositories
 
     `repo` is the cubicweb repository (not a vcs repository)
@@ -85,13 +85,17 @@ def import_content(session, commitevery=10):
         try:
             repohdlr = repository_handler(vcsrepo)
         except VCSException, ex:
+            if raise_on_error:
+                raise
             session.error(str(ex))
             continue
         import_vcsrepo_content(session, repohdlr, vcsrepo,
-                               commitevery=commitevery)
+                               commitevery=commitevery,
+                               raise_on_error=raise_on_error)
 
 
-def import_vcsrepo_content(session, repohdlr, vcsrepo, commitevery=10):
+def import_vcsrepo_content(session, repohdlr, vcsrepo, commitevery=10,
+                           raise_on_error=False):
     """synchronize content of given vcs repository"""
     try:
         repohdlr.import_content(vcsrepo, commitevery)
@@ -99,6 +103,8 @@ def import_vcsrepo_content(session, repohdlr, vcsrepo, commitevery=10):
         # it back afterwards
         session.commit()
     except:
+        if raise_on_error:
+            raise
         # clear in cases new files have been added in cache, but entities will
         # be rollbacked
         repohdlr.vf_path_cache.clear()
@@ -157,7 +163,7 @@ def set_at_revision(session, reid, safetybelt=False):
         reid, reid, reid, reid)
     if safetybelt:
         sql += (' AND NOT EXISTS(SELECT 1 FROM at_revision_relation AS atr '
-                'WHERE atr.eid_from=%s AND atr.eid_to=VC.cw.cw_eid)') % reid
+                'WHERE atr.eid_to=%s AND atr.eid_from=VC.cw_eid)') % reid
     session.system_sql(sql)
 
 
