@@ -201,9 +201,9 @@ class Revision(AnyEntity):
 
     def dc_title(self):
         if self.changeset:
-            return '#%(rev)s:%(changeset)s' % {
+            return u'#%(rev)s:%(changeset)s' % {
                 'rev': self.revision, 'changeset': self.changeset}
-        return '#%s' % self.revision
+        return u'#%s' % self.revision
 
     def dc_long_title(self):
         try:
@@ -224,6 +224,14 @@ class Revision(AnyEntity):
     def repository(self):
         return self.from_repository[0]
 
+    @property
+    def file_revisions(self):
+        return self.reverse_from_revision
+
+    @property
+    def children_revisions(self):
+        return self.reverse_parent_revision
+
 
 class VersionedFile(AnyEntity):
     """customized class for VersionedFile entities"""
@@ -233,7 +241,7 @@ class VersionedFile(AnyEntity):
     # XXX branches
     def dc_title(self):
         if self.deleted_in_branch():
-            return '%s (%s)' % (self.path, self._cw._('DELETED'))
+            return u'%s (%s)' % (self.path, self._cw._('DELETED'))
         return self.path
 
     def dc_long_title(self):
@@ -259,7 +267,7 @@ class VersionedFile(AnyEntity):
     @property
     def path(self):
         if self.directory:
-            return '%s/%s' % (self.directory, self.name)
+            return u'%s/%s' % (self.directory, self.name)
         return self.name
 
     @property
