@@ -2,7 +2,7 @@
 """hooks for vcsfile content types
 
 :organization: Logilab
-:copyright: 2007-2010 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+:copyright: 2007-2011 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 :contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
 """
 __docformat__ = "restructuredtext en"
@@ -14,11 +14,17 @@ import shutil
 from logilab.mtconverter import need_guess, guess_mimetype_and_encoding
 
 from cubicweb import QueryError, ValidationError, Binary
+from cubicweb.selectors import objectify_selector
 from cubicweb.server import hook
 from cubicweb.selectors import is_instance, score_entity, configuration_values
 
 from cubes.vcsfile import IMMUTABLE_ATTRIBUTES, bridge
 
+# XXX in cw >= 3.12.4
+@objectify_selector
+def is_user_session(cls, req, **kwargs):
+    return not req.is_internal_session
+
 def repo_cache_dir(config):
     directory = config['local-repo-cache-root']
     if not osp.isabs(directory):
@@ -260,14 +266,11 @@ class AddVersionContentHook(hook.Hook):
 
 class AddDeletedVersionContentHook(hook.Hook):
     __regid__ = 'vcsfile.add_deleted_version_content_hook'
-    __select__ = hook.Hook.__select__ & is_instance('DeletedVersionContent')
+    __select__ = (hook.Hook.__select__ & is_instance('DeletedVersionContent')
+                  & is_user_session())
     events = ('before_add_entity',)
 
     def __call__(self):
-        # skip further processing if the revision is being imported from the
-        # vcs repository
-        if self._cw.is_internal_session:
-            return
         vcsrepohdlr, transaction = self.entity._vc_prepare()
         vf = self.entity._vc_vf()
         vcsrepohdlr.add_versioned_file_deleted_content(self._cw, transaction, vf,
@@ -276,14 +279,11 @@ class AddDeletedVersionContentHook(hook.Hook):
 
 class AddVersionedFileHook(hook.Hook):
     __regid__ = 'vcsfile.add_versioned_file_hook'
-    __select__ = hook.Hook.__select__ & is_instance('VersionedFile')
+    __select__ = (hook.Hook.__select__ & is_instance('VersionedFile')
+                  & is_user_session())
     events = ('before_add_entity',)
 
     def __call__(self):
-        # skip further processing if the revision is being imported from the
-        # vcs repository
-        if self._cw.is_internal_session:
-            return
         CheckVersionedFileOp(self._cw, entity=self.entity)
 
 
@@ -410,8 +410,9 @@ class CheckRevisionOp(hook.Operation):
 
 class CheckImmutableAttributeHook(hook.Hook):
     __regid__ = 'vcsfile.check_immutable_attribute_hook'
-    __select__ = hook.Hook.__select__ & is_instance(
-        'Revision', 'DeletedVersionContent', 'VersionContent')
+    __select__ = (hook.Hook.__select__
+                  & is_instance('Revision', 'DeletedVersionContent', 'VersionContent')
+                  & is_user_session())
     events = ('before_update_entity',)
 
     def __call__(self):
