@@ -1,7 +1,7 @@
 """repository abstraction for a mercurial repository
 
 :organization: Logilab
-:copyright: 2007-2010 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+:copyright: 2007-2011 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 :contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
 """
 
@@ -16,7 +16,9 @@ from datetime import datetime
 from mercurial import hg, ui, revlog, context, util, commands as hgcmd
 from mercurial.node import nullid, short as short_hex
 
-from cubicweb import Binary, QueryError
+from logilab.common import tempattr
+
+from cubicweb import Binary, QueryError, ExecutionError
 from cubicweb.server.ssplanner import EditedEntity
 
 from cubes.vcsfile import repo, bridge
@@ -94,8 +96,8 @@ class HGRepository(repo.VCSRepository):
         except Exception, ex:
             self.exception('Impossible to open repository %s (%s)', self, ex)
             return
-        # actual rev is the *number* of revision, latest rev the highest
-        # revision number while hg revisions start at 0 -> - 1
+        # `actualrev` is the *number* of revision, `latestrev` the highest
+        # revision number while hg revisions start at 0 , hence the - 1
         actualrev = bw_nb_revisions(repo.changelog) - 1
         session = repoentity._cw
         latestrev = repoentity.latest_known_revision()
@@ -112,7 +114,6 @@ class HGRepository(repo.VCSRepository):
         self.info('hg repo %s: known revisions up to %s, repo revision %s',
                   self.path, latestrev, actualrev)
         for i in xrange(latestrev + 1, actualrev + 1):
-            self.info('importing revision %s', i)
             try:
                 self.import_revision(repoentity, repo, i)
             except:
@@ -126,6 +127,7 @@ class HGRepository(repo.VCSRepository):
                 session.set_pool()
 
     def import_revision(self, repoentity, repo, i):
+        self.info('importing revision %s', i)
         session = repoentity._cw
         execute = session.execute
         node = repo.changelog.node(i)
@@ -144,7 +146,6 @@ class HGRepository(repo.VCSRepository):
                    'changeset': changeset,
                    'branch': unicode(ctx.branch()),
                    }
-        self.imported_revision = i # for hooks
         parents = [short_hex(n) for n in repo.changelog.parents(node)
                    if n != nullid]
         if parents:
@@ -168,34 +169,8 @@ class HGRepository(repo.VCSRepository):
             upath = unicode(path, self.encoding, 'replace')
             if not (path_filter is None or upath.startswith(path_filter)):
                 continue
-            directory, fname = split(path)
-            session.transaction_data['vcs_importing'] = self.eid, directory, fname, i
-            filectx = ctx[path]
-            vcdata = {'data': Binary(filectx.data())}
-            renameinfo = filectx.renamed()
-            if renameinfo:
-                oldfile, fileid = renameinfo
-                pfctx = repo.filectx(oldfile, fileid=fileid)
-                pcs = short_hex(pfctx.node())
-                dir, name = split(unicode(oldfile, self.encoding, 'replace'))
-                pvc = execute('VersionContent X WHERE '
-                              'X from_revision REV, REV changeset %(cs)s, '
-                              'REV from_repository R, R eid %(r)s, '
-                              'X content_for VF, VF directory %(dir)s, VF name %(name)s',
-                              {'cs':  pcs, 'r': self.eid,
-                               'dir': dir, 'name': name})
-                if pvc:
-                    assert len(pvc) == 1
-                    if oldfile in removed:
-                        vcdata['vc_rename'] = pvc[0][0]
-                    else:
-                        vcdata['vc_copy'] = pvc[0][0]
-                else:
-                    self.error('detected copy or rename of %s@%s but unable'
-                               ' to find associated version content',
-                               oldfile, pcs)
-            bridge.import_version_content(session, self.eid, reveid, upath,
-                                          date, **vcdata)
+            self._import_version_content(session, i, reveid, removed,
+                                         path, upath, date, repo, ctx)
         for path in removed:
             upath = unicode(path, self.encoding, 'replace')
             if not (path_filter is None or upath.startswith(path_filter)):
@@ -204,6 +179,40 @@ class HGRepository(repo.VCSRepository):
                                                   upath, date)
         bridge.set_at_revision(session, reveid)
 
+    def _import_version_content(self, session, i, reveid, removed, path, upath,
+                                date, repo, ctx):
+        directory, fname = split(path)
+        session.transaction_data['vcs_importing'] = self.eid, directory, fname, i
+        filectx = ctx[path]
+        vcdata = {'data': Binary(filectx.data())}
+        renameinfo = filectx.renamed()
+        if renameinfo:
+            pvc, oldfile = self._renamed_vc_rset(session, repo, renameinfo)
+            if pvc:
+                assert len(pvc) == 1
+                if oldfile in removed:
+                    vcdata['vc_rename'] = pvc[0][0]
+                else:
+                    vcdata['vc_copy'] = pvc[0][0]
+            else:
+                self.error('detected copy or rename of %s@%s but unable'
+                           ' to find associated version content',
+                           oldfile, pcs)
+        bridge.import_version_content(session, self.eid, reveid, upath,
+                                      date, **vcdata)
+
+    def _renamed_vc_rset(self, session, repo, renameinfo):
+        oldfile, fileid = renameinfo
+        pfctx = repo.filectx(oldfile, fileid=fileid)
+        pcs = short_hex(pfctx.node())
+        dir, name = split(unicode(oldfile, self.encoding, 'replace'))
+        return session.execute('VersionContent X WHERE '
+                               'X from_revision REV, REV changeset %(cs)s, '
+                               'REV from_repository R, R eid %(r)s, '
+                               'X content_for VF, VF directory %(dir)s, VF name %(name)s',
+                               {'cs':  pcs, 'r': self.eid,
+                                'dir': dir, 'name': name}), oldfile
+
     def hgrepo(self):
         return hg.repository(ui.ui(), self.path)
 
@@ -234,6 +243,142 @@ class HGRepository(repo.VCSRepository):
         transaction.changes[self.encode(vf.path)] = _DELETED
         return vf.path
 
+    # reimport #################################################################
+
+    def check(self, repoentity, start_revision=0, commitevery=1):
+        """import content from the mercurial repository"""
+        assert self.eid == repoentity.eid
+        if not exists(self.path):
+            self.info('repository %s not accessible in %s',
+                      repoentity.dc_title(), self.path)
+            return
+        try:
+            repo = self.hgrepo()
+        except Exception, ex:
+            self.exception('Impossible to open repository %s (%s)', self, ex)
+            return
+        # actual rev is the *number* of revision, latest rev the highest
+        # revision number while hg revisions start at 0 -> - 1
+        actualrev = bw_nb_revisions(repo.changelog) - 1
+        session = repoentity._cw
+        latestrev = repoentity.latest_known_revision()
+        self.info('hg repo %s: known revisions up to %s, repo revision %s',
+                  self.path, latestrev, actualrev)
+        for i in xrange(start_revision, actualrev + 1):
+            if i > latestrev:
+                self.import_revision(repoentity, repo, i)
+            else:
+                self.check_revision(repoentity, repo, i)
+            if not i % commitevery:
+                # give a change to threads waiting for a pool by freeing it then
+                # reacquiring it
+                session.commit()
+                session.set_pool()
+        session.commit()
+
+    def check_revision(self, repoentity, repo, i):
+        self.info('checking revision %s', i)
+        session = repoentity._cw
+        execute = session.execute
+        node = repo.changelog.node(i)
+        changeset = unicode(short_hex(node))
+        rev = execute('Any X WHERE X from_repository R, R eid %(repo)s, X changeset %(cs)s',
+                      {'repo': repoentity.eid, 'cs': changeset})
+        if len(rev) > 1:
+            # will crash on entities referencing this revision has parent,
+            # better to stop now
+            raise ExecutionError('revision #%s has more than one related entities (%s). '
+                                 'Fix this manually first' % (
+                                     (changeset, ','.join(str(e) for e, in rev))))
+        if not rev:
+            rev = execute('Any X WHERE X from_repository R, R eid %(repo)s, X revision %(rev)s, X changeset ULL',
+                          {'repo': repoentity.eid, 'rev': i})
+            if rev:
+                self.warning('revision missing changeset: %s (#%s)', i, changeset)
+                rev.get_entity(0, 0).set_attributes(changeset=changeset)
+        if not rev:
+            self.warning('revision %s isn\'t imported', changeset)
+            self.import_revision(repoentity, repo, i)
+            return
+        rev = rev.get_entity(0, 0)
+        ctx = repo.changectx(i)
+        date = datetime.fromtimestamp(ctx.date()[0])
+        # XXX check revision's attributes
+        # revdata = {'date': date, 'revision': i,
+        #            'author': unicode(ctx.user(), self.encoding, 'replace'),
+        #            'description': unicode(ctx.description(), self.encoding, 'replace'),
+        #            'changeset': changeset,
+        #            'branch': unicode(ctx.branch()),
+        #            }
+        parents = [short_hex(n) for n in repo.changelog.parents(node)
+                   if n != nullid]
+        if parents:
+            if len(parents) == 1:
+                parentsrset = execute('Revision X WHERE X changeset %(cs)s, '
+                                  'X from_repository R, R eid %(r)s',
+                                  {'cs': parents[0], 'r': self.eid})
+            else:
+                parentsrset = execute('Revision X WHERE X changeset IN (%s), '
+                                  'X from_repository R, R eid %%(r)s'
+                                  % ','.join("'%s'"%cs for cs in parents),
+                                  {'r': self.eid})
+            assert len(parentsrset) == len(parents), 'missing parent: expected %s, got %s' % (
+                parents, [e.changeset for e in parentsrset.entities()])
+            for preveid, in parentsrset:
+                if execute('SET R parent_revision PR WHERE R eid %(r)s, PR eid %(pr)s, '
+                           'NOT R parent_revision PR',
+                           {'r': rev.eid, 'pr': preveid}):
+                    self.warning('add missing parent revision')
+        if not repoentity.import_revision_content:
+            return
+        changes = repo.status(ctx.parents()[0].node(), ctx.node())[:3]
+        modified, added, removed = changes
+        path_filter = repoentity.subpath
+        for path in modified + added:
+            upath = unicode(path, self.encoding, 'replace')
+            if not (path_filter is None or upath.startswith(path_filter)):
+                continue
+            directory, fname = split(path)
+            vc = execute('Any VC WHERE VF directory %(d)s, VF name %(n)s, '
+                         'VC is VersionContent, VC content_for VF, VC from_revision R, R eid %(r)s',
+                         {'d': directory, 'n': fname, 'r': rev.eid})
+            if not vc:
+                self.warning('add missing file %s modified/added by revision', upath)
+                self._import_version_content(session, i, rev.eid, removed,
+                                             path, upath, date, repo, ctx)
+            else:
+                filectx = ctx[path]
+                renameinfo = filectx.renamed()
+                if renameinfo:
+                    pvc, oldfile = self._renamed_vc_rset(session, repo, renameinfo)
+                    if pvc:
+                        assert len(pvc) == 1
+                        if oldfile in removed:
+                            if execute('SET X vc_rename Y WHERE X eid %(x)s, Y eid %(y)s, NOT X vc_rename Y',
+                                       {'x': vc[0][0], 'y': pvc[0][0]}):
+                                self.warning('add rename information on %s', upath)
+                        else:
+                            if execute('SET X vc_copy Y WHERE X eid %(x)s, Y eid %(y)s, NOT X vc_copy Y',
+                                       {'x': vc[0][0], 'y': pvc[0][0]}):
+                                self.warning('add copy information on %s', upath)
+                    else:
+                        self.error('detected copy or rename of %s@%s but unable'
+                                   ' to find associated version content',
+                                   oldfile, pcs)
+        for path in removed:
+            upath = unicode(path, self.encoding, 'replace')
+            if not (path_filter is None or upath.startswith(path_filter)):
+                continue
+            directory, fname = split(path)
+            vc = execute('Any VC WHERE VF directory %(d)s, VF name %(n)s, '
+                         'VC is DeletedVersionContent, VC content_for VF, VC from_revision R, R eid %(r)s',
+                         {'d': directory, 'n': fname, 'r': rev.eid})
+            if not vc:
+                self.warning('add missing file %s deleted by revision', upath)
+                bridge.import_deleted_version_content(session, self.eid, rev.eid,
+                                                      upath, date)
+        bridge.set_at_revision(session, rev.eid, safetybelt=True)
+
 
 class HGTransaction(object):
     def __init__(self, repohdlr, session, revision):
@@ -241,7 +386,7 @@ class HGTransaction(object):
         self.session = session
         self.revision = revision
         self.repo = repohdlr.hgrepo()
-        self._wlock = self.repo.wlock()
+        # see http://mercurial.selenic.com/wiki/LockingDesign
         self._lock = self.repo.lock()
         self.extra = {}
         self.changes = {}
@@ -315,36 +460,26 @@ class HGTransaction(object):
         # / rql repository)
         source = self.session.repo.system_source
         # remove previous cached value to only save the changeset
-        with setattrcm(self.revision, 'cw_edited', EditedEntity(self.revision)) as rev:
+        with tempattr(self.revision, 'cw_edited', EditedEntity(self.revision)) as rev:
             rev.cw_edited['changeset'] = unicode(short_hex(node))
             source.update_entity(self.session, rev)
+        # we need to explicitly commit the connection to the database since this
+        # method is actually called from a postcommit event, i.e. once pool has
+        # already been commited
+        self.session.pool.connection('system').commit()
         # XXX restore entity's dict?
         self.rollback()
 
     def rollback(self):
         try:
-            # mercurial >= 1.3.1
-            self._wlock.release()
-            self._lock.release()
-        except AttributeError:
-            del self._wlock
-            del self._lock
-
-
-
-class setattrcm(object):
-    def __init__(self, obj, attr, value):
-        self.obj = obj
-        self.attr = attr
-        self.value = value
-
-    def __enter__(self):
-        self.old = getattr(self.obj, self.attr)
-        setattr(self.obj, self.attr, self.value)
-        return self.obj
-
-    def __exit__(self, exctype, value, traceback):
-        setattr(self.obj, self.attr, self.old)
+            from mercurial.lock import release
+            release(self._lock)
+        except ImportError:
+            try:
+                # mercurial >= 1.3.1
+                self._lock.release()
+            except AttributeError:
+                del self._lock
 
 
 from logging import getLogger
