@@ -65,6 +65,11 @@ class Revision(EntityType):
         'update': ('managers', 'owners'),
         'delete': (),
         }
+    __unique_together__ = [('from_repository', 'revision'),
+                           # XXX behaviour vs NULL values depending on the
+                           # backend? NULL values are not considered equals by
+                           # sqlite nor postgres at least
+                           ('from_repository', 'changeset')]
     # NOTE: creation_date will be set at revision date
     revision = Int(required=True, indexed=True,
                    description=_('revision number.'),
@@ -88,6 +93,7 @@ class Revision(EntityType):
 
 class VersionedFile(EntityType):
     """a file stored in a versioned source"""
+    __unique_together__ = [('from_repository', 'directory', 'name')]
     __permissions__ = {
         'read':   ('managers', 'users', 'guests'),
         'add':    ('managers', 'users',),
@@ -96,7 +102,7 @@ class VersionedFile(EntityType):
         }
     directory = String(required=True, indexed=True,
                        description=_('directory of the file in the repository.'),
-                   __permissions__=RO_ATTRPERMS)
+                       __permissions__=RO_ATTRPERMS)
     name = String(required=True, fulltextindexed=True,
                   description=_('name of the file in the repository.'),
                    __permissions__=RO_ATTRPERMS)
