@@ -224,14 +224,14 @@ class VcsRmForm(form.FormViewMixIn, EntityView):
                     {'file': entity.view('incontext')})
         self.w(u'</div>')
         # XXX branch
-        form = self.vreg['forms'].select('base', self._cw, rset=self.rset,
-                                         form_renderer_id='base', entity=entity,
-                                         action=self.build_url('vcsrm'),
-                                         form_buttons=[fw.SubmitButton()])
+        form = self._cw.vreg['forms'].select('base', self._cw, rset=self.cw_rset,
+                                             form_renderer_id='base', entity=entity,
+                                             action=self._cw.build_url('vcsrm'),
+                                             form_buttons=[fw.SubmitButton()])
         form.append_field(ff.StringField(name='msg', eidparam=True,
                                          value=u'',
                                          label=_('commit message')))
-        self.w(form.render())
+        form.render(w=self.w)
 
 
 class VcsRmController(controller.Controller):
